{% extends "joyous/joyous_base.html" %}
{% load static wagtailcore_tags i18n %}

{% block content %}
<div class="content">
  <div class="joy-title joy-title--page">
    <h1>{{ page.title }}</h1>
  </div>
  {% block cal_all %}
  {% block cal_intro %}
  {{ page.intro|richtext }}
  {% endblock cal_intro %}
  {% block view_choices %}
  {% endblock view_choices %}
  {% block cal_subtitle %}
  {% endblock cal_subtitle %}
  {% block cal_events %}
  <div data-version="{{ version }}"></div>
  {% endblock cal_events %}
  {% block cal_footer %}<div class="joy-export">
    <a href="{{ request.path }}?format=ical">{% trans "Export iCal" %}</a>
  </div>{% endblock cal_footer %}
  {% endblock cal_all %}
</div>
{% endblock %}
