{% extends "joyous/joyous_base.html" %}
{% load wagtailcore_tags joyous_tags %}

{% block content %}
<div class="content">
  <div class="joy-title joy-title--page">
    <h1>{{ page.title }}</h1>
  </div>
  {{ page.content|richtext }}
  {% group_upcoming_events %}
</div>
{% endblock %}
