{% extends "base.html" %}
{% load static %}

{% block body_class %}template-{{ page.get_verbose_name|slugify }}{% endblock %}

{% block extra_css %}
  <link rel="stylesheet" type="text/css" href="{% static 'joyous/css/joyous.css' %}">
  {% if themeCSS %}<link rel="stylesheet" type="text/css" href="{{ themeCSS }}">{% endif %}
{% endblock %}

{% block content %}
{% endblock %}

{% block extra_js %}
{% endblock %}
