from django.conf.locale import LANG_INFO
LANG_INFO['en-nz'] = {'bidi': False,
                      'code': 'en-nz',
                      'name': 'New Zealand English',
                      'name_local': 'Kiwi English'}

