{% extends "joyous/calendar_list_base.html" %}
{% load wagtailcore_tags i18n %}

{% block view_choices %}
{% include "joyous/includes/events_view_choices.html" %}
{% endblock view_choices %}

{% block cal_view_class %}joy-cal-list--day{% endblock %}

{% block cal_subtitle %}
<div class="joy-title joy-title--list">
  <h2>{% blocktrans with day=day|date:"l, jS F Y" %}Events on {{ day }}{% endblocktrans %}</h2>
</div>
{% endblock cal_subtitle %}
